/**
	This file is part of Hedge Maze.

    Hedge Maze is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Hedge Maze is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Hedge Maze.  If not, see <http://www.gnu.org/licenses/>.

	Copyright 2006, 2010 James Hill
*/

#include "DIinput.h"

// the constructor will currently do nothing
DIinput::DIinput()
{

}

// the destructor will release the objects if they are active
DIinput::~DIinput()
{
	delete ifd;	// delete the memory used for the input structure

	if(DI_mouse != NULL)	// release the mouse
		DI_mouse->Release();

	if(DI_keyboard != NULL) // release the keyboard
		DI_keyboard->Release();

	if(DI_object != NULL)	// release the object
		DI_object->Release();
}

// the initDI function will attempt to initialize the Direct Input objects
// and devices.  It will return false if initialization fails
bool DIinput::initDI(HINSTANCE hInstance, HWND wndHandle)
{

	// allocate memory for the input from device structure
	ifd = new inputFromDevice;

	// create the input object
	if(FAILED(DirectInput8Create( hInstance, DIRECTINPUT_VERSION, IID_IDirectInput8, (void**)&DI_object, NULL )))
		return false;

	// create the device
	if(FAILED(DI_object->CreateDevice(GUID_SysKeyboard, &DI_keyboard, NULL)))
		return false;

	// set the data formate
	if(FAILED(DI_keyboard->SetDataFormat(&c_dfDIKeyboard)))
		return false;

	// Set the cooperative level
	if(FAILED(DI_keyboard->SetCooperativeLevel(wndHandle, DISCL_FOREGROUND | DISCL_NONEXCLUSIVE)))
		return false;

	// aquire the device
	if(FAILED(DI_keyboard->Acquire()))
		return false;

	// create the mouse
	if(FAILED(DI_object->CreateDevice(GUID_SysMouse, &DI_mouse, NULL)))
        return false;
	
	// set the data formate
	if(FAILED(DI_mouse->SetDataFormat(&c_dfDIMouse2)))
        return false;

	// set the coop level
	if( FAILED(DI_mouse->SetCooperativeLevel(wndHandle , DISCL_FOREGROUND | DISCL_EXCLUSIVE)))
		return false;

	// aquire the device
	if(FAILED(DI_mouse->Acquire()))
		return false;
	
	return true;
}

inputFromDevice * DIinput::getInput()
{
	ZeroMemory(ifd->keyboardBuffer,sizeof(ifd->keyboardBuffer));

	DI_mouse->GetDeviceState( sizeof(ifd->mouseBuffer), & ifd->mouseBuffer );
	
	DI_keyboard->GetDeviceState(sizeof(ifd->keyboardBuffer),ifd->keyboardBuffer);

	return ifd;
}