/**
	This file is part of Hedge Maze.

    Hedge Maze is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Hedge Maze is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Hedge Maze.  If not, see <http://www.gnu.org/licenses/>.

	Copyright 2006, 2010 James Hill
*/

#ifndef DIINPUT_H
#define DIINPUT_H

#include <windows.h>
#include <dinput.h>

#define DIRECTINPUT_VERSION 0x0800
#define KEYDOWN(name,key) (name[key] & 0x80) 

// this structure will allow for the return of 
// the input data in an easy to access package
struct inputFromDevice{
	bool keyboard;
	bool mouse;
	DIMOUSESTATE2 mouseBuffer;
	char keyboardBuffer[256];
};

// the actual input class which will return input data
// to the calling function
class DIinput
{
public:

	// default constructor is all that's needed
	DIinput();

	// default constructor will release all DI objects and devices
	~DIinput();

	// the init function will attempt to initialized all devices
	// if it can not initialize them, it will return false
	bool initDI(HINSTANCE hInstance, HWND wndHandle);

	// the get input function will return a pointer to its
	// inputFromDevice structure after filling it with any
	// new input data
	inputFromDevice * getInput();

private:

	inputFromDevice * ifd;	// node input from the device

	LPDIRECTINPUT8				DI_object;		// the DirectInput object
	LPDIRECTINPUTDEVICE8		DI_keyboard;	// the keyboard
	LPDIRECTINPUTDEVICE8		DI_mouse;		// the mouse
};

#endif //DIINPUT_H