/**
	This file is part of Hedge Maze.

    Hedge Maze is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Hedge Maze is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Hedge Maze.  If not, see <http://www.gnu.org/licenses/>.

	Copyright 2006, 2010 James Hill
*/

#include "graphicsManager.h"

// the constructor will do nothing
graphicsManager::graphicsManager()
{

}

// the destructor will release the memory
graphicsManager::~graphicsManager()
{
	topWall->Release();
	botWall->Release();
	rightWall->Release();
	leftWall->Release();
	D3D_device->Release();
	D3D_object->Release();
}

// initGM will set up the object and the device
// it will aslo initialize the vertex buffer for each of the walls
// and the floor.  Loading of the textures is also done here.
bool graphicsManager::initGM(unsigned short w, unsigned short h, HWND wndHandle)
{
	// create the Direct3D object
	D3D_object = Direct3DCreate9(D3D_SDK_VERSION);

	// if failed, return false
	if(D3D_object == NULL) 
		return false;

	// fill the parameters list
	D3DPRESENT_PARAMETERS d3dpp;
	d3dpp.BackBufferWidth = w;
	d3dpp.BackBufferHeight = h;
	d3dpp.BackBufferFormat = D3DFMT_A8R8G8B8;
	d3dpp.BackBufferCount = 1;
	d3dpp.MultiSampleType = D3DMULTISAMPLE_NONE;
	d3dpp.MultiSampleQuality = 0;
	d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
	d3dpp.hDeviceWindow = wndHandle;
	d3dpp.Windowed = false;
	d3dpp.EnableAutoDepthStencil = true;
	d3dpp.AutoDepthStencilFormat = D3DFMT_D24S8;
	d3dpp.Flags = 0;
	d3dpp.FullScreen_RefreshRateInHz = D3DPRESENT_RATE_DEFAULT;
	d3dpp.PresentationInterval = D3DPRESENT_INTERVAL_IMMEDIATE;

	if(FAILED(D3D_object->CreateDevice(D3DADAPTER_DEFAULT,
								  D3DDEVTYPE_HAL,
								  wndHandle,
								  D3DCREATE_HARDWARE_VERTEXPROCESSING,
								  &d3dpp,
								  &D3D_device)))
		  return false;


	D3D_device -> SetRenderState ( D3DRS_MULTISAMPLEANTIALIAS , TRUE );
	D3D_device -> SetRenderState ( D3DRS_CULLMODE , D3DCULL_NONE );
	D3D_device -> SetRenderState ( D3DRS_FILLMODE, D3DFILL_SOLID);


	/////////////////////////////////////////////////////////////////
	//// initialize the verticies of the walls
	/////////////////////////////////////////////////////////////////

	VOID * pVert;
	
	// the top wall
	vertex twall [ ] = {
		{ MAZEDIM * -1, MAZEDIM *  1, MAZEDIM * 1, 0 , 0, -1, 0, 0 },
		{ MAZEDIM *  1, MAZEDIM *  1, MAZEDIM * 1, 0 , 0, -1, 1, 0 },
		{ MAZEDIM * -1, MAZEDIM * -1, MAZEDIM * 1, 0 , 0, -1, 0, 1 },

		{ MAZEDIM *  1, MAZEDIM * -1, MAZEDIM * 1, 0 , 0, -1, 1, 1 },
		{ MAZEDIM * -1, MAZEDIM * -1, MAZEDIM * 1, 0 , 0, -1, 0, 1 },
		{ MAZEDIM *  1, MAZEDIM *  1, MAZEDIM * 1, 0 , 0, -1, 1, 0 },
	};
	
	// set up the vertex buffer
	if ( FAILED ( D3D_device ->CreateVertexBuffer ( sizeof ( twall ) ,
														0,
														D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_TEX1,
														D3DPOOL_DEFAULT,
														& topWall,
														NULL ) ) )
	{
		return false;
	}

	if ( FAILED ( topWall->Lock ( 0 , 0 , ( void ** ) & pVert , 0 ) ) )
	{
		return false;
	}

	memcpy ( pVert , twall , sizeof ( twall ) );

	topWall ->Unlock ( );

	// the bottom wall
	vertex bwall [ ] = {
		{ MAZEDIM * -1, MAZEDIM *  1, MAZEDIM * -1, 0 , 0, 1, 0, 0 },
		{ MAZEDIM *  1, MAZEDIM *  1, MAZEDIM * -1, 0 , 0, 1, 1, 0 },
		{ MAZEDIM * -1, MAZEDIM * -1, MAZEDIM * -1, 0 , 0, 1, 0, 1 },

		{ MAZEDIM *  1, MAZEDIM * -1, MAZEDIM * -1, 0 , 0, 1, 1, 1 },
		{ MAZEDIM * -1, MAZEDIM * -1, MAZEDIM * -1, 0 , 0, 1, 0, 1 },
		{ MAZEDIM *  1, MAZEDIM *  1, MAZEDIM * -1, 0 , 0, 1, 1, 0 },
	};
	
	// set up the vertex buffer
	if ( FAILED ( D3D_device ->CreateVertexBuffer ( sizeof ( bwall ) ,
														0,
														D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_TEX1,
														D3DPOOL_DEFAULT,
														& botWall,
														NULL ) ) )
	{
		return false;
	}

	if ( FAILED ( botWall->Lock ( 0 , 0 , ( void ** ) & pVert , 0 ) ) )
	{
		return false;
	}

	memcpy ( pVert , bwall , sizeof ( bwall ) );

	botWall ->Unlock ( );

	// The left wall
	vertex lwall [ ] = {
		{ MAZEDIM * -1, MAZEDIM *  1, MAZEDIM *  1, 1, 0, 0, 1, 0 },
		{ MAZEDIM * -1, MAZEDIM *  1, MAZEDIM * -1, 1, 0, 0, 0, 0 },
		{ MAZEDIM * -1, MAZEDIM * -1, MAZEDIM * -1, 1, 0, 0, 0, 1 },

		{ MAZEDIM * -1, MAZEDIM *  1, MAZEDIM *  1, 1, 0, 0, 1, 0 },
		{ MAZEDIM * -1, MAZEDIM * -1, MAZEDIM *  1, 1, 0, 0, 1, 1 },
		{ MAZEDIM * -1, MAZEDIM * -1, MAZEDIM * -1, 1, 0, 0, 0, 1 },
	};
	
	// set up the vertex buffer
	if ( FAILED ( D3D_device ->CreateVertexBuffer ( sizeof ( lwall ) ,
														0,
														D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_TEX1,
														D3DPOOL_DEFAULT,
														& leftWall,
														NULL ) ) )
	{
		return false;
	}

	if ( FAILED ( leftWall->Lock ( 0 , 0 , ( void ** ) & pVert , 0 ) ) )
	{
		return false;
	}

	memcpy ( pVert , lwall , sizeof ( lwall ) );

	botWall ->Unlock ( );

	// The right wall
	vertex rwall [ ] = {
		{ MAZEDIM * 1, MAZEDIM *  1, MAZEDIM *  1, -1, 0, 0, 1, 0 },
		{ MAZEDIM * 1, MAZEDIM *  1, MAZEDIM * -1, -1, 0, 0, 0, 0 },
		{ MAZEDIM * 1, MAZEDIM * -1, MAZEDIM * -1, -1, 0, 0, 0, 1 },

		{ MAZEDIM * 1, MAZEDIM *  1, MAZEDIM *  1, -1, 0, 0, 1, 0 },
		{ MAZEDIM * 1, MAZEDIM * -1, MAZEDIM *  1, -1, 0, 0, 1, 1 },
		{ MAZEDIM * 1, MAZEDIM * -1, MAZEDIM * -1, -1, 0, 0, 0, 1 },
	};
	
	// set up the vertex buffer
	if ( FAILED ( D3D_device ->CreateVertexBuffer ( sizeof ( rwall ) ,
														0,
														D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_TEX1,
														D3DPOOL_DEFAULT,
														& rightWall,
														NULL ) ) )
	{
		return false;
	}

	if ( FAILED ( rightWall->Lock ( 0 , 0 , ( void ** ) & pVert , 0 ) ) )
	{
		return false;
	}

	memcpy ( pVert , rwall , sizeof ( rwall ) );

	rightWall ->Unlock ( );

	// The floor
	vertex mfloor [ ] = {
		{ MAZEDIM * -1, MAZEDIM * -1, MAZEDIM * -1, 0, 1, 0, 0, 0 },
		{ MAZEDIM *  1, MAZEDIM * -1, MAZEDIM * -1, 0, 1, 0, 1, 0 },
		{ MAZEDIM * -1, MAZEDIM * -1, MAZEDIM *  1, 0, 1, 0, 0, 1 },

		{ MAZEDIM * 1, MAZEDIM * -1, MAZEDIM * -1, 0, 1, 0, 1, 0 },
		{ MAZEDIM * 1, MAZEDIM * -1, MAZEDIM *  1, 0, 1, 0, 1, 1 },
		{ MAZEDIM *-1, MAZEDIM * -1, MAZEDIM *  1, 0, 1, 0, 0, 1 },
	};
	
	// set up the vertex buffer
	if ( FAILED ( D3D_device ->CreateVertexBuffer ( sizeof ( mfloor ) ,
														0,
														D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_TEX1,
														D3DPOOL_DEFAULT,
														& floor,
														NULL ) ) )
	{
		return false;
	}

	if ( FAILED ( floor->Lock ( 0 , 0 , ( void ** ) & pVert , 0 ) ) )
	{
		return false;
	}

	memcpy ( pVert , mfloor , sizeof ( mfloor ) );

	rightWall ->Unlock ( );

	// initialize the textures
	HRESULT hedgeLoad = D3DXCreateTextureFromFile ( D3D_device , TEXT("hedge.bmp") , &mazeWall );
	if ( FAILED ( hedgeLoad ) )
	{
		if(hedgeLoad == D3DERR_NOTAVAILABLE)
			return true;
		else if(hedgeLoad == D3DERR_OUTOFVIDEOMEMORY)
			return false;
		else if(hedgeLoad == D3DERR_INVALIDCALL)
			return true;
		else if(hedgeLoad == D3DXERR_INVALIDDATA)
			return false;

		return false;
	}

	if ( FAILED ( D3DXCreateTextureFromFile ( D3D_device , TEXT("path.bmp") , &mazeFloor ) ) )
	{
		return false;
	}

	// set up the camera projection
	D3DXMATRIXA16 proj;
	D3DXMatrixPerspectiveFovLH ( & proj , D3DX_PI / 4 , 4.0f/3.0f , .5f , 60*MAZEDIM );
	D3D_device->SetTransform ( D3DTS_PROJECTION , & proj );

	// material
	D3DMATERIAL9 material;

	ZeroMemory ( & material , sizeof ( D3DMATERIAL9 ) );

	material.Diffuse.r = material.Ambient.r = 1;
	material.Diffuse.g = material.Ambient.g = 1;
	material.Diffuse.b = material.Ambient.b = 1;
	material.Diffuse.a = material.Ambient.a = 1;

	D3D_device ->SetMaterial ( & material );
	
	return true;
}

// the draw function will draw the screen
void graphicsManager::draw(maze * theMaze, player * thePlayer)
{
	// maze properties
	unsigned char w, h;

	D3DXMATRIXA16 view;
	D3DXMATRIXA16 translation;
	D3DXVECTOR3 position = thePlayer->getPosition();
	D3DXVECTOR3 look = thePlayer->getLookAt();
	D3DXVECTOR3 up(0,1,0);
	D3DXMatrixLookAtLH ( & view , & position , & look , & up );
	D3D_device ->SetTransform ( D3DTS_VIEW , & view );

	D3D_device ->SetRenderState( D3DRS_AMBIENT, 0x00505050 );

	D3D_device->Clear(0,0, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB ( 0 , 0 , 150 ), 1.0f, 0);

	if(SUCCEEDED(D3D_device->BeginScene()))
	{
		D3D_device ->SetFVF ( D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_TEX1 );
		
		// draw each node and then the floor
		unsigned char mw = theMaze->getWidth();
		unsigned char mh = theMaze->getHeight();

		for(w = 0; w < mw; w++) 
		{
			for(h = 0; h < mh; h++)
			{
				D3DXMatrixTranslation(&translation, MAZEDIM*2*w ,0,MAZEDIM*2*h );
				D3D_device ->SetTransform ( D3DTS_WORLD , &translation );

				// set the texture to the wall texture
				// and draw each wall if needed
				// this algorithm only draws the left and top walls of each node
				// the bottom and right size of the maze are only drawn if
				// at those bounds
				D3D_device ->SetTexture(0, mazeWall);
				if(theMaze->getNode(w,h)->top==true) {
					D3D_device ->SetStreamSource ( 0 , topWall , 0 , sizeof ( vertex ) );
					D3D_device ->DrawPrimitive ( D3DPT_TRIANGLELIST , 0 , 2 );
				}
				if(theMaze->getNode(w,h)->left==true){
					D3D_device ->SetStreamSource ( 0 , leftWall , 0 , sizeof ( vertex ) );
					D3D_device ->DrawPrimitive ( D3DPT_TRIANGLELIST , 0 , 2 );
				}
				if(w == mw-1){
					D3D_device ->SetStreamSource ( 0 , rightWall , 0 , sizeof ( vertex ) );
					D3D_device ->DrawPrimitive ( D3DPT_TRIANGLELIST , 0 , 2 );
				}
				if(h == 0) {
					D3D_device ->SetStreamSource ( 0 , botWall , 0 , sizeof ( vertex ) );
					D3D_device ->DrawPrimitive ( D3DPT_TRIANGLELIST , 0 , 2 );
				}
				D3D_device->SetTexture(0,mazeFloor);
				D3D_device->SetStreamSource ( 0, floor, 0, sizeof ( vertex));
				D3D_device->DrawPrimitive( D3DPT_TRIANGLELIST, 0, 2);
			}
		}
		D3D_device ->EndScene ( );
	}
	D3D_device->Present(0,0,0,0);
}