/**
	This file is part of Hedge Maze.

    Hedge Maze is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Hedge Maze is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Hedge Maze.  If not, see <http://www.gnu.org/licenses/>.

	Copyright 2006, 2010 James Hill
*/

#ifndef GRAPHICSMANAGER_H
#define GRAPHICSMANAGER_H

#include <windows.h>
#include <d3d9.h>
#include <d3dx9.h>
#include "maze.h"
#include "player.h"

#define MAZEDIM	10		// this determins how much we multipy by the unit maze node
						// to get a larger node size

// define the vetext formate
////////////// test code //////////////////
struct vertex
{
		FLOAT x , y , z;
		FLOAT nx , ny , nz;
		FLOAT u , v;
};

class graphicsManager
{
public:

	// the constructor will do nothing
	graphicsManager();

	// the destructor will release the D3D object and device
	~graphicsManager();

	// the init function will initialize the graphic manager
	// to the specified resolution.  If anything fails to
	// initialize, the function will return false
	bool initGM(unsigned short int w, unsigned short int h, HWND wndHandle);

	// we will eventally need a texture loading funciton

	// the draw function will eventually take a pointer to the maze
	// and a pointer to the camera (player) and will draw the maze
	// as quickly as it possibly can
	void draw(maze * theMaze, player * thePlayer);

private:

	// the object and device
	IDirect3D9 *				D3D_object;	// the Direct 3D object
	IDirect3DDevice9 *			D3D_device;	// the Direct 3D device

	// four vertext vectors will be created
	// one for each possible wall of the maze
	IDirect3DVertexBuffer9 *	topWall;
	IDirect3DVertexBuffer9 *	botWall;
	IDirect3DVertexBuffer9 *	rightWall;
	IDirect3DVertexBuffer9 *	leftWall;
	IDirect3DVertexBuffer9 *	floor;

	// Eventually we will need a buffer to the texture to place on the maze
	LPDIRECT3DTEXTURE9		mazeWall;
	LPDIRECT3DTEXTURE9		mazeFloor;
	LPDIRECT3DTEXTURE9		mazeTop;

};


#endif //GRAPHICSMANAGER_H