/**
	This file is part of Hedge Maze.

    Hedge Maze is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Hedge Maze is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Hedge Maze.  If not, see <http://www.gnu.org/licenses/>.

	Copyright 2006, 2010 James Hill
*/

#include <windows.h>
#include "graphicsManager.h"
#include "DIinput.h"
#include "timerClass.h"

HINSTANCE hInst;
HWND wndHandle;

HRESULT						hr;	

graphicsManager * theManager;
player * thePlayer;
maze * theMaze;
/*bool movet;
bool moveb;
bool mover;
bool movel;*/

LRESULT CALLBACK WndProc ( HWND hWnd , UINT message , WPARAM wParam , LPARAM lParam )
{

	switch ( message )
	{
		case WM_DESTROY:
			PostQuitMessage ( WM_QUIT );
			break;
		case WM_TIMER:
			theManager->draw(theMaze, thePlayer);
			break;
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
	}

	return DefWindowProc ( hWnd , message , wParam , lParam );
}

bool initWindow (unsigned short w, unsigned short h, HINSTANCE hInstance )
{

	WNDCLASSEX wcex;

	wcex.cbSize			= sizeof ( WNDCLASSEX );
	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= 0;
	wcex.hCursor		= LoadCursor ( NULL , NULL );
	wcex.hbrBackground	= ( HBRUSH ) ( COLOR_WINDOW + 1 );
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= LPCWSTR("DirectXExample");
	wcex.hIconSm		= 0;

	RegisterClassEx ( & wcex );

	wndHandle = CreateWindow ( LPCWSTR("DirectXExample"),
							   LPCWSTR("DirectXExample") , 
							   WS_OVERLAPPEDWINDOW,
							   //WS_EX_TOPMOST | WS_POPUP | WS_VISIBLE,
							   CW_USEDEFAULT ,
							   CW_USEDEFAULT ,
							   w ,
							   h 
							   , NULL ,
							   NULL ,
							   hInstance ,
							   NULL );

	if ( ! wndHandle )
		return false;

	ShowWindow ( wndHandle , SW_SHOW );
	UpdateWindow ( wndHandle );

	return true;
}

// a function to determin whether or not a move is valid
void checkMove(double timerDelta, int dir)
{
	// determin what node the player is in
	float dt, db, dr, dl;
	bool t, b, r, l;
	float dx = thePlayer->getPosition().x;
	float dz = thePlayer->getPosition().z;
	float olx= thePlayer->getLookAt().x;
	float olz= thePlayer->getLookAt().z;
	float nx;
	float nz;
	float nlx;
	float nlz;
	unsigned char w = (dx+MAZEDIM)/(MAZEDIM*2);
	unsigned char h = (dz+MAZEDIM)/(MAZEDIM*2);

	t = theMaze->getNode(w,h)->top;
	b = theMaze->getNode(w,h)->bot;
	r = theMaze->getNode(w,h)->right;
	l = theMaze->getNode(w,h)->left;

	dt = h*2*MAZEDIM+MAZEDIM;
	db = h*2*MAZEDIM-MAZEDIM;
	dr = w*2*MAZEDIM+MAZEDIM;
	dl = w*2*MAZEDIM-MAZEDIM;

	switch(dir) {
		case 0:
			thePlayer->transForward(3*MAZEDIM*timerDelta,true,true);
			break;
		case 1:
			thePlayer->transForward(-3*MAZEDIM*timerDelta,true,true);
			break;
		case 2:
			thePlayer->transRight(-3*MAZEDIM*timerDelta,true,true);
			break;
		case 3:
			thePlayer->transRight(3*MAZEDIM*timerDelta,true,true);
			break;

	}
	
	nx = thePlayer->getPosition().x;
	nz = thePlayer->getPosition().z;
	nlx = thePlayer->getLookAt().x;
	nlz = thePlayer->getLookAt().z;

	if(t) {
		if(nz > dt - 5) {
			thePlayer->setpz(dz);
			thePlayer->setlz(olz);
		}
	}
	if(b) {
		if(nz < db + 5) {
			thePlayer->setpz(dz);
			thePlayer->setlz(olz);
		}
	}
	if(r) {
		if(nx > dr - 5) {
			thePlayer->setpx(dx);
			thePlayer->setlx(olx);
		}
	}
	if(l){
		if(nx < dl + 5) {
			thePlayer->setpx(dx);
			thePlayer->setlx(olx);
		}
	}
}

int WINAPI WinMain ( HINSTANCE hInstance , HINSTANCE hPrevInstance , LPSTR lpCmdLine , int nCmdShow )
{
	// initialize window
	unsigned short w = 1024;
	unsigned short h = 768;

	if ( !initWindow (w, h, hInstance ) )
		return false;
	
	double timerDelta;					// hold what is returned by the timer
	timerClass theTimer;				// the timer class
	theManager = new graphicsManager();	// the graphics manager
	thePlayer = new player();			// the player
	theMaze = new maze(20,20);			// the maze 20 x 20
	thePlayer->setPosition((theMaze->getWidth()/2)*MAZEDIM*2,0);	// set the position to the half the maze width
	thePlayer->setLookat((theMaze->getWidth()/2)*MAZEDIM*2,1);		// set the look at to look down the positive z axis
	inputFromDevice * ifd;				// an input from device pointer
	DIinput * theInput = new DIinput(); // the input device

	theManager->initGM(w,h,wndHandle);	// initializes the graphics manager
	theInput->initDI(hInstance, wndHandle);	// initializes the input device
	theMaze->initMaze();				// initializes the maze
	theMaze->outputASCIImaze();			// outputs the created maze to a text file

	theTimer.addDelta();

	SetTimer( wndHandle, 0, 10, NULL );	// sets the timer to go off every 

	MSG msg;
	ZeroMemory ( & msg , sizeof ( msg ) );
	while ( msg.message != WM_QUIT )
	{
		ifd = theInput->getInput();

		timerDelta = CONVERT_TO_SECONDS(theTimer.getDelta(0));

		if(KEYDOWN(ifd->keyboardBuffer,DIK_W))
			checkMove(timerDelta+0.000001,0);
			
		if(KEYDOWN(ifd->keyboardBuffer,DIK_S))
			checkMove(timerDelta+0.000001,1);
			
		if(KEYDOWN(ifd->keyboardBuffer,DIK_D))
			checkMove(timerDelta+0.000001,2);
			
		if(KEYDOWN(ifd->keyboardBuffer,DIK_A))
			checkMove(timerDelta+0.000001,3);
			
		if(KEYDOWN(ifd->keyboardBuffer,DIK_ESCAPE))
			PostQuitMessage(WM_QUIT);
		else
			theTimer.updateDelta(0);

		if(ifd->mouseBuffer.lX != 0)
			thePlayer->rotate(-1*ifd->mouseBuffer.lX*0.005);

		if ( PeekMessage ( & msg , NULL , 0 , 0 , PM_REMOVE ) )
		{
			TranslateMessage ( & msg );
			DispatchMessage ( & msg );
		}
	}

	delete theMaze;
	delete theInput;
	delete thePlayer;
	delete theManager;

	return true;
}