/**
	This file is part of Hedge Maze.

    Hedge Maze is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Hedge Maze is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Hedge Maze.  If not, see <http://www.gnu.org/licenses/>.

	Copyright 2006, 2010 James Hill
*/

#ifndef MAZE_H
#define MAZE_H

#include <fstream>	// used to output an ascii version of the maze
#include <stdlib.h>	// used for the rand function
#include <time.h>

union mazeNode {
	struct {
		bool top :1;
		bool bot :1;
		bool right :1;
		bool left :1;
		bool eptop :1;
		bool epbot :1;
		bool epright :1;
		bool epleft :1;
	};
	char node;
};

class maze 
{
public:
	// constructors
	maze();													// default constructor
	maze(unsigned char width, unsigned char height);		// overloaded constructor
	maze(const maze& toCopy);								// copy constructor
	
	// destructor
	~maze();

	// Access as if it were a multidymentional array
	mazeNode * getNode(unsigned char x, unsigned char y) const;

	// create the maze
	void createMaze(unsigned char nx, unsigned char ny, unsigned char entry);

	// getters
	unsigned char getHeight() const;						// returns the height
	unsigned char getWidth() const;							// returns the width
	mazeNode * getMaze();									// returns a pointer to the nodes
	
	// setters
	void setHeight(unsigned char newHeight);				// set the height
	void setWidth(unsigned char newWidth);					// set the width
	
	// outputs
	void outputASCIImaze();									// outputs the maze to an ascii file

	// maze creation functions
	bool isAvailable (unsigned char nx, unsigned char ny);			// check to see if node is available
	unsigned char choosePath (unsigned char nx, unsigned char ny);	// chooses a random path to follow
	void initMaze();												// initialize the maze
	void fillWalls();												// fills in the walls of the maze


private:
	
	unsigned char mazeHeight;								// the height of the maze
	unsigned char mazeWidth;								// the width of the maze
	mazeNode * theMaze;										// will hold a dynamically created maze of size mazeWidth x mazeHeight
};

#endif //MAZE_H
