/**
	This file is part of Hedge Maze.

    Hedge Maze is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Hedge Maze is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Hedge Maze.  If not, see <http://www.gnu.org/licenses/>.

	Copyright 2006, 2010 James Hill
*/

#include "player.h"

// the constructor will do nothing
player::player()
{
	// initializes the vectors
	// orients it looking down the z axis with
	// the negative x axis as it's right vector
	px = 0; pz = 0;
	lx = 0; lz = 1;
	rx = -1;rz = 0;
	fx = 0; fz = 1;
}

// the destructor will free the alocated memory
player::~player()
{

}

// setting the position 
void player::setPosition(float x, float z)
{
	px = x;
	pz = z;
}

// setting the look vector
void player::setLookat(float x, float z)
{
	lx = x;
	lz = z;
}

// set the right vector
void player::setRight(float x, float z)
{
	rx = x;
	rz = z;
}

// set the front vector
void player::setFront(float x, float z)
{
	fx = x;
	fz = z;
}

// returns a pointer to the position vector
D3DXVECTOR3 player::getPosition()
{
	D3DXVECTOR3 returnVal(px, 0, pz);
	return returnVal;
}

// returns a pointer to the look at vector
D3DXVECTOR3 player::getLookAt()
{
	D3DXVECTOR3 returnVal(lx, 0, lz);
	return returnVal;
}

// the translation functions
void player::transForward(float amount, bool xdir, bool zdir)
{
	// add the amount multiplied by
	// the forward vector to the
	// position and look at vectors
	
	if(xdir)
	{
		px += fx * amount;
		lx += fx * amount;
	}

	if(zdir)
	{
		pz += fz * amount;
		lz += fz * amount;
	}
}

void player::transRight(float amount, bool xdir, bool zdir)
{
	// add the amount multipled by
	// the right vector to the
	// position and look at vectors
	
	if(xdir) 
	{
		px += rx * amount;
		lx += rx * amount;
	}
	
	if(zdir) 
	{
		pz += rz * amount;
		lz += rz * amount;
	}

}

// The rotation funciton
void player::rotate(float angle)
{
	// Rotate the right vector
	float tx, tz;
	float co, si;

	co = cos(angle);
	si = sin(angle);

	tx = rx*co-rz*si;
	tz = rx*si+rz*co;

	rx = tx;
	rz = tz;

	// Rotate the front vector
	tx = fx*co-fz*si;
	tz = fx*si+fz*co;

	// translate the look at vector by the 
	lx = lx + (tx-fx);
	lz = lz + (tz-fz);

	// set the forward vector to the new value
	fx = tx;
	fz = tz;
}
