/**
	This file is part of Hedge Maze.

    Hedge Maze is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Hedge Maze is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Hedge Maze.  If not, see <http://www.gnu.org/licenses/>.

	Copyright 2006, 2010 James Hill
*/

#include "timerClass.h"

// the constructor basically initiates the current time
// the current time will be updated on every call to
// any function
//
// James Hill
// Jan-15-2006
timerClass::timerClass()
{	
	// set current time
	currentTime = clock();

	// set the number of deltas to zero
	numTimeDelta = 0;
}

// the destructor clears the vector of previous times.
// this is probably redundant but I'll do it anyway
//
// James Hill
// Jan-15-2005
timerClass::~timerClass()
{
	// clear out the vector
	prevTimes.clear();
}

// return the current time as a double
//
// James Hill
// Jan-15-2006
double timerClass::getCurrentTime()
{
	// update the current time
	currentTime = clock();

	// return it as a double
	return (double)currentTime;
}

// add delta will add a time delta to the current 
// vector of times
//
// James Hill
// Jan-15-2006
void timerClass::addDelta()
{
	// the current time is updated on every funcition call
	currentTime = clock();
	
	// push that time back into the delta buffer
	prevTimes.push_back(currentTime);
	numTimeDelta++;
}

// remove delta will remove the given delta
// form the vector of times
//
// James Hill
// Jan-15-2006
void timerClass::removeDelta(unsigned int delta)
{

	// decrement the number of deltas
	numTimeDelta--;

	// copy previouse deltas to the current
	for(;delta<numTimeDelta;delta++)
		prevTimes[delta]=prevTimes[delta+1];
}

// get Delta returns the difference between the current
// time and the time stored at the specified 
// delta
//
// James Hill
// Jan-16-2005
double timerClass::getDelta(unsigned int delta)
{
	clock_t returnValue = -1;

	// update the current time
	currentTime = clock();

	if(delta < numTimeDelta)
	{
		returnValue = currentTime - prevTimes[delta];
	}

	return (double)returnValue;
}

// update delta will update the current delta to 
// reflect the current time
//
// James Hill
// Jan-15-2006
void timerClass::updateDelta(unsigned int delta)
{
	// update the current time
	currentTime = clock ();

	if(delta < numTimeDelta)
		prevTimes[delta]=currentTime;
}