/**
	This file is part of Hedge Maze.

    Hedge Maze is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Hedge Maze is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Hedge Maze.  If not, see <http://www.gnu.org/licenses/>.

	Copyright 2006, 2010 James Hill
*/

#ifndef	TIMERCLASS_H
#define TIMERCLASS_H

#include <vector>
#include <time.h>

// this define will allow for easy conversion to 
// time in seconds.
#define CONVERT_TO_SECONDS(x) x/CLOCKS_PER_SEC

class timerClass 
{
public:

	// the constructor will set the current time
	timerClass();
	~timerClass();

	// get current time will return the current value
	// of the current time variable
	double getCurrentTime();

	// add delta will push the current time into the
	// vector
	void addDelta();
	
	// remove delta will remove the given delta
	void removeDelta(unsigned int delta);

	// get delta returns the difference between the current
	// time and the given delta.
	double getDelta(unsigned int delta);

	// update delta will update the delta value to reflect
	// the current time
	void updateDelta(unsigned int delta);

private:

	// the number of deltas in the vector
	unsigned int numTimeDelta;	

	// the total time that has elapsed
	clock_t currentTime;		 
	
	// a container for all previouse time deltas
	std::vector<clock_t> prevTimes;
};

#endif TIMERCLASS_H